﻿(function ($, ns, undefined) {
    var addVersionDialog = function (config) {
        config.dialogClass = "manage-versions-dialog";
        config.dialogSize = "medium thin";
        config.loadControllerPath = 'version/addversiondialog';
        config.loadControllerData = {
            mediaItemId: config.mediaItemId
        };
        var self = this;
        config.actions = {
            saveAndUseAsCurrent: {
                text: translate("common.Save"),
                cssClass:"ok-action",
                action: function() {
                    self.saveAndUseAsCurrent();
                },
                disabled: true
            },
            cancel: {
                text: translate("common.Cancel"),
                action: function() {
                    self.cancel();
                },
                noButton: true,
                cssClass: "cancel-action"
            }

        }
        this.init(config);
        return this;
    };
    //setup inheritance
    addVersionDialog.prototype = new ImageVaultUi.Dialog;
    addVersionDialog.constructor = addVersionDialog;
    addVersionDialog.prototype.base = ImageVaultUi.Dialog.prototype;

    //own methods
    addVersionDialog.prototype.removeUploadedVersion = function (callback) {
        var self = this;
        if (self.uploadedMediaVersion) {
            var id = self.uploadedMediaVersion.Id;
            console.log("[Cancel] Removing uploaded version " + id);
            app.json({
                url: "version/deleteversion",
                data: { versionId: id },
                error: function (err) {
                    alert("Failed to remove uploaded media version: " + err);
                    callback();
                },
                success: function () {
                    self.uploadedMediaVersion = null;
                    callback();
                }
            });
        } else {
            callback();
        }

    };
    addVersionDialog.prototype.cancel = function () {
        var self = this;
        self.removeUploadedVersion(function() {
            self.close();
        });
    };

    addVersionDialog.prototype.saveAndUseAsCurrent = function() {
        var self = this;
        var id = this.uploadedMediaVersion.Id;
        app.json({
            url: "version/save",
            data: { versionId: id, setCurrent:true, comment: self.$comment.val() },
            error: function (err) {
                alert("Failed to Save version: " + err);
            },
            success: function () {
                console.log("Saved new version as current " + id);
                //notify photostream of the changes
                app.trigger('media:versionchange');
                self.close();
            }
        });

    },
    ///Method updates the uploaded media version thumbnail
    addVersionDialog.prototype.loadUploadedVersionThumbnail = function (mediaVersion) {
        if (!this.isOpened()) return; //don't load thumbs if we have closed the window.
        var self = this;
        app.json("version/getmediaversionthumbnail", { versionId: mediaVersion.Id }, function (thumbnail) {
            if (thumbnail) {
                self.$nevVersionThumbnail.attr("src", "");
                self.$nevVersionThumbnail.attr("width", thumbnail.Width);
                self.$nevVersionThumbnail.attr("height", thumbnail.Height);
                self.$nevVersionThumbnail.attr("src", thumbnail.Url);
                self.$nevVersionThumbnail.error(function () {
                    ImageVaultUi.ThumbnailLoader.HandleError(self.$nevVersionThumbnail[0]);
                });
            } else {
                self.loadUploadedVersionThumbnailTimeout = setTimeout(function () {
                    self.loadUploadedVersionThumbnailTimeout = null;
                    self.loadUploadedVersionThumbnail(mediaVersion);
                }, 2000);
            }
        });
    };
    addVersionDialog.prototype.onClose = function () {
        //clear timeouts if dialog is closed
        if (this.loadUploadedVersionThumbnailTimeout) {
            clearTimeout(this.loadUploadedVersionThumbnailTimeout);
        }
    };
    addVersionDialog.prototype.onLoad = function ($dialog) {
        var self = this;
        self.$uploadArea = $('#uploadArea', $dialog);
        self.$nevVersionThumbnail = $("#new_version_thumbnail", self.$uploadArea);
        self.$uploadProgress = $('#upload_progress', self.$uploadArea);
        self.$message = $('#message', self.$uploadArea);
        self.$uploadErrorMessage = $('.upload_error_message', self.$message);
        self.$uploadArea.bind('dragover', function (e) {
            // Only change class if we're draggin files over it
            if ($.inArray('Files', e.originalEvent.dataTransfer.types) > -1) {
                $(e.currentTarget).addClass('dragover');
            }
        });

        self.$uploadArea.bind('dragleave', function (e) {
            $(e.currentTarget).removeClass('dragover');
        });

        self.$comment = $("textarea#Comment", $dialog);

        function log() {
            var str = "";

            plupload.each(arguments, function (arg) {
                var row = "";

                if (typeof (arg) != "string") {
                    plupload.each(arg, function (value, key) {
                        // Convert items in File objects to human readable form
                        if (arg instanceof plupload.File) {
                            // Convert status to human readable
                            switch (value) {
                                case plupload.QUEUED:
                                    value = 'QUEUED';
                                    break;
                                case plupload.UPLOADING:
                                    value = 'UPLOADING';
                                    break;
                                case plupload.FAILED:
                                    value = 'FAILED';
                                    break;
                                case plupload.DONE:
                                    value = 'DONE';
                                    break;
                            }
                        }

                        if (typeof (value) != "function") {
                            row += (row ? ', ' : '') + key + '=' + value;
                        }
                    });

                    str += row + " ";
                } else {
                    str += arg + " ";
                }
            });
            console.log(str);
            //$('#log').val($('#log').val() + str + "\r\n");
        }

        var uploader = new plupload.Uploader({
            browse_button: self.$uploadArea.attr("id"),
            drop_element: self.$uploadArea.attr("id"),
            // General settings
            runtimes: 'html5,flash,html4',
            url: app.createPath('upload/Plupload/'),
            //max_file_size: '10mb',
            chunk_size: '3mb',
            max_file_count: 1,
            //we use the normal names since we have problem with html5 and chunked transfers
            //that uses the filename blob in the uploadedfile["name"]
            unique_names: false,
            //allows us to upload more once the upload has finished
            multiple_queues: false,
            // Flash/Silverlight paths
            flash_swf_url: app.createPath('plupload/js/plupload.flash.swf'),
            silverlight_xap_url: app.createPath('plupload/js/plupload.silverlight.xap'),

            // PreInit events, bound before any internal events
            preinit: {
                Init: function (up, info) {
                    log('[Init]', 'Info:', info, 'Features:', up.features);
                },

                UploadFile: function (up, file) {
                    log('[UploadFile]', file);
                    up.settings.multipart_params = { mediaItemId: self.mediaItemId };
                }
            },

            // Post init events, bound after the internal events
            init: {
                Refresh: function (up) {
                    // Called when upload shim is moved
                    log('[Refresh]');
                },

                StateChanged: function (up) {
                    // Called when the state of the queue is changed
                    log('[StateChanged]', up.state == plupload.STARTED ? "STARTED" : "STOPPED");
                },

                QueueChanged: function (up) {
                    // Called when the files in queue are changed by adding/removing files
                    log('[QueueChanged]');
                },

                UploadProgress: function (up, file) {
                    // Called while a file is being uploaded
                    self.$uploadProgress.text(file.percent + " %");
                    self.$uploadProgress.width(file.percent);
                    log('[UploadProgress]', 'File:', file, "Total:", up.total);
                },

                FilesAdded: function (up, files) {
                    self.removeUploadedVersion(function() {
                        self.$uploadErrorMessage.text("");
                        var maxCount = up.settings.max_file_count;
                        var exceedingCount = files.length - maxCount;
                        if (maxCount > 0 && exceedingCount > 0) {
                            for (var i = 1; i <= exceedingCount; i++) {
                                up.removeFile(files[files.length - i]);
                            }
                            alert('The maximum number of files is ' + up.settings.max_file_count + '.');
                        }
                        // Called when files are added to queue
                        log('[FilesAdded]');

                        plupload.each(files,
                            function(file) {
                                log('  File:', file);
                            });
                        self.$uploadProgress.show();
                        self.$message.hide();
                        self.$uploadArea.removeClass("dragover");
                        up.start();
                    });
                },

                FilesRemoved: function (up, files) {
                    // Called when files where removed from queue
                    log('[FilesRemoved]');

                    plupload.each(files, function (file) {
                        log('  File:', file);
                    });
                },

                FileUploaded: function (up, file, info) {
                    // Called when a file has finished uploading
                    log('[FileUploaded] File:', file, "Info:", info);
                    var mediaVersion = JSON.parse(info.response);
                    self.$nevVersionThumbnail.show();
                    self.$uploadProgress.hide();
                    self.uploadedMediaVersion = mediaVersion;
                    self.loadUploadedVersionThumbnail(mediaVersion);
                    self.enableAction("saveAndUseAsCurrent");
                },

                ChunkUploaded: function (up, file, info) {
                    // Called when a file chunk has finished uploading
                    log('[ChunkUploaded] File:', file, "Info:", info);
                },

                Error: function (up, args) {
                    // Called when a error has occured
                    self.$uploadProgress.hide();
                    self.$uploadErrorMessage.text(translate('versionsdialog.FailMessage'));
                    self.$uploadErrorMessage.show();
                    self.$uploadArea.removeClass('dragover');
                    self.$message.show();
                    // Handle file specific error and general error
                    if (args.file) {
                        log('[errorx]', args, "File:", args.file);
                    } else {
                        log('[errorx]', args);
                    }
                },
                UploadComplete: function (up, file) {
                    $('a.organize').removeClass('inactive');
                }
            }
        });
        uploader.init();
    };


    //export symbol
    ns.AddVersionDialog = addVersionDialog;
})(jQuery, window.ImageVaultUi = window.ImageVaultUi || {});