﻿(function ($, ns) {
    var scope = function (config) {
        this.init(config);
        return this;
    };
    scope.prototype = {
        test: null,
        init: function (config) {
            var self = this;
            //Event on hash change
            $(window).bind('hashchange', function () {
                self.initScope();
            });
            //Trigger hashchange event on load.
            $(window).trigger('hashchange');
            //Event for removing filter
            $('#scope a.remove').live('click', function () {
                self.removeFilter($(this).closest('li'));
                return false;
            });
            $('#scope a.remove-all').live('click', function () {
                $.bbq.removeState('categories');
                $.bbq.removeState('q');
                return false;
            });
        },
        initScope: function () {
            var self = this;
            self.updateLocation($.bbq.getState('vaultid'));
            self.updateCategories($.bbq.getState('categories'));
            self.updateSearchString($.bbq.getState('q'));
            if ($('#scope ul li').length > 1 && $('#scope a.remove-all').length == 0) {
                $('#scope-clear-anchor').tmpl().appendTo($('#scope'));
            }
            if ($('#scope ul li').length <= 1) {
                $('#scope a.remove-all').remove();
            }
        },
        updateLocation: function (location) {
            //Update current location (vault, collection etc)
            //TODO: Only supports vaults...
            var location = $("#vaults a[data-vault-id=" + location + "]").text();
            $('#scope h1').html(location);
        },
        updateCategories: function (categoriesAsString) {
            //Rebuild list of categories in current scope
            //Remove current
            $('#scope ul li[class=category]').remove();
            if (typeof categoriesAsString === "undefined" || categoriesAsString.length == 0) {
                $('#nav-tabs .categories').parent().removeClass('has-selected');
                return;
            }

            $('#nav-tabs .categories').parent().addClass('has-selected');

            var categories = categoriesAsString.split('e');
            //If scope is empty create new list
            if ($('#scope ul').length == 0) {
                $('#scope').append('<ul></ul>');
            }
            //Iterate categories from url fragment
            for (var i in categories) {
                //Get category name from list of categories based on id
                var categoryName = $('#categories li[data-category-id="' + categories[i] + '"] > label').text();
                //Add category to scope
                $('#scope-template').tmpl([{ Name: categoryName, Class: "category", Id: categories[i]}]).appendTo($('#scope ul'));
            }
        },
        updateSearchString: function (searchString) {
            //Update search in current scope
            //Remove current
            $('#scope ul li[class=search]').remove();
            if (typeof searchString === "undefined" || searchString.length == 0) {
                return;
            }
            //If scope is empty create new list
            if ($('#scope ul').length == 0) {
                $('#scope').append('<ul></ul>');
            }
            //Add search to scope
            $('#scope-template').tmpl([{ Name: searchString, Class: "search"}]).prependTo($('#scope ul'));
        },
        removeFilter: function ($li) {
            //Search, category or filter (future).
            var type = $li.attr('class');
            if (type == 'search') {
                $.bbq.removeState('q');
            }
            if (type == 'category') {
                var categories = $.bbq.getState('categories').split('e');
                //Remove category from array.
                for (var i = 0; i < categories.length; i++) {
                    if (categories[i] == $li.attr('data-id')) {
                        categories.splice(i, 1);
                        break;
                    }
                }
                //Remove key if no categories left
                if (categories.length == 0) {
                    $.bbq.removeState('categories');
                    return;
                }
                //Update url state
                var state = {};
                state['categories'] = categories.join('e');
                $.bbq.pushState(state);
            }
            $li.remove();
        }
    };
    ns.Scope = scope;
})(jQuery, window.ImageVaultUi = window.ImageVaultUi || {});