﻿(function ($, ns, undefined) {
	var shareDialog = function (config) {
	    var self = this;
        config.dialogSize = "small thin";
	    config.dialogClass = "share-dialog";
	    config.hideHrBeforeActions = true;

        config.loadControllerPath = "share/new";
        config.loadControllerData = { mediaFormatId: config.mediaFormatId, mediaItemId: config.mediaItemIds };

	    config.actions = {
	        next: {
	            text: translate("common.Next"),
	            action: function() {
	                $('#share-form').submit();
	            },
	            cssClass: "submit-action",
	            disabled: true
	        },
	        cancel: {
	            text: translate("common.Cancel"),
	            action: function() {
                    self.close();
	            },
	            noButton: true,
	            cssClass: "cancel-action"
	        },
            closeWindow: {
                text: translate("sharedialog.CloseThisWindow"),
                action: function() {
                    self.close();
                },
                cssClass: "cancel-action",
                hidden: true
            },
	        // Actions for step 2, hidden by default
            copyLink: {
                text: translate("sharedialog.CopyLink"),
                action: function() {},
                cssClass: "ok-action clipboard-text",
                hidden: true
            }
	    };

		this.init(config);
		return this;
	};

    //setup inheritance
	shareDialog.prototype = new ImageVaultUi.Dialog;
	shareDialog.constructor = shareDialog;
	shareDialog.prototype.base = ImageVaultUi.Dialog.prototype;

    shareDialog.prototype.onLoad = function($dialog) {
        var self = this;
        $("#ShareTitle", this.$dialog).focus();
        this.copyLinkManualPopup = new ImageVaultUi.TemplatedPopup({
            templateString: ImageVaultUi.TemplatedPopup.tplCopyLinkManualPopup
        });

        $("#ShareTitle", this.$dialog).bind("change keyup paste", function(e) {
           if ($(e.currentTarget).val()) {
               $(".actions .submit-action", self.$dialog).removeClass("disabled");
           } else {
               $(".actions .submit-action", self.$dialog).addClass("disabled");
           }
        });
    };

    shareDialog.prototype.onError = function(data) {
        try {
            alert(this.saveErrorMsg + "\n" + data.responseText.substr(data.responseText.indexOf('</html>') + 7));
        } catch (err) {
            alert(this.saveErrorMsg);
        }
    };

    shareDialog.prototype.createShareComplete = function(data) {
        //this.closeDialog();
        var notif = new IVNotification(translate("sharedialog.SaveSuccess"),
        {
            duration: 2000
        });
        $("#edit-details", this.$dialog).hide();
        this.hideAction("next");
        this.hideAction("cancel");
        var jResultDiv = $("#show-result", this.$dialog);
        var jPMessage = $("#message", jResultDiv);
        var msg = data.htmlMessage;
        var absUrl = ImageVault.Client.prototype._makeAbsoluteUrl(data.url);
        msg = msg.replace(/\{AbsoluteUrl\}/, absUrl);
        $('div.actions a.clipboard-text').attr('data-clipboard-text', absUrl);
        jPMessage.html(msg);
        jResultDiv.show();
        this.showAction("copyLink");
        this.showAction("closeWindow");
        //notify which images was shared
        app.trigger("share:created", this.mediaItemIds);
        ////Update vault list
        //app.trigger("shareDialog:vaultSaved", data);

        this.initClipboard();
    };

    shareDialog.prototype.initClipboard = function() {
        if (!app.hasFlash()) {
            // we do not have flash, fallback to popup instead of zeroclipboard
            $('a.clipboard-text')
                .click($.proxy(function(e) {
                        // Todo: change alert to design
                        var link = $('a.clipboard-text').attr('data-clipboard-text');
                        //alert('Flash is not installed so here is the link for your copy-pasting: ' + link);

                        copyLinkManualValues.actualLink = $('a.clipboard-text').attr('data-clipboard-text');

                        this.copyLinkManualPopup.injectAfter('div.overlay',
                            copyLinkManualValues,
                            function(html) {
                                // Hook up the eventhandlers for the popup
                                $('a.close-action')
                                    .click(function(e) {
                                        $('div#manual-copy')
                                            .fadeOut('fast',
                                                function() {
                                                    $(this).remove();
                                                });
                                        e.preventDefault();
                                    });
                                $('div#manual-copy').fadeIn('fast');
                                $('div#manual-copy input#share-link-input').focus().select();
                            });
                        e.preventDefault();
                    },
                    this));
        }
        // Load the ZeroClipboard script if it's not loaded
        else if (!window.ZeroClipboard /* && !window.clipboardData //Dont use IE impl.*/) {
            $.getScript('Scripts/ZeroClipboard.min.js',
                $.proxy(function() {
                        ZeroClipboard.config({
                            moviePath: app.createPath("Scripts/ZeroClipboard.swf")
                        });
                        this._attachClipboardHandlers();
                    },
                    this));
        } else {
            this._attachClipboardHandlers();
        }
    };

    shareDialog.prototype._attachClipboardHandlers = function() {
        if (window.clipboardData && clipboardData.setData && false /*Dont use IE impl.*/) {
            $("a.clipboard-text")
                .live("click",
                    $.proxy(function(evt) {
                            clipboardData.setData('text', $(this).attr('data-clipboard-text'));
                            var clipboardMsg = new IVNotification(this.msgCopiedToClipboard,
                            { showOverlay: false, duration: 4000 });
                        },
                        this));
        } else {
            // Attach the clipboard-function to all anchors...
            this.clipboardClient = new ZeroClipboard($("a.clipboard-text"));
            // Hook up an eventhandler that shows a tooltip message when the link is copied correctly
            this.clipboardClient.addEventListener('complete',
                $.proxy(function(client, text) {
                        var clipboardMsg = new IVNotification(translate("sharedialog.LinkWasCopiedToClipboard"),
                        { showOverlay: false, duration: 4000 });
                    },
                    this));
        }
    };

	ns.ShareDialog = shareDialog;
})(jQuery, window.ImageVaultUi = window.ImageVaultUi || {});