﻿(function ($, ns, undefined) {
    var infoPanel = Backbone.Model.extend({
        //properties
        currentId: null,
        $domobject: null,
        $viewport: null,
        $noItemSelected: null,
        //methods
        initialize: function () {
            this.$domobject = this.get('$domobject');

            if (this.$domobject === undefined)
                throw "$domobject is not defined in config";
            this.$viewport = $("#details-viewport #scrollable", this.$domobject);
            if (this.$viewport.length == 0)
                throw "Cannot find tab div in $domobject";
            this.$noItemSelected = $('#details-no-mediaitem-selected', this.$domobject);
            if (this.$noItemSelected.length == 0)
                throw "Cannot find $noItemSelected in $domobject";

            var self = this;

            app.bind("deleteDialog:fileDeleted", this.fileDeleted, this);

            this.updateSelectedTab();

            //initialization code here
            //Resize nav height on window resize 
            $(window).resize(function () {
                self.initViewport();
            });
            $("li a", this.$domobject).click(function () {
                //ignore events if the no items selected is visible...
                if ($(':visible', self.$noItemSelected).length > 0) {
                    return false;
                }
                var newTab = $(this).attr("data-name");
                self.loadTab(newTab);
                return false;
            });
            app.bind("media:versionchange", this.refresh, this);
        },
        //refreshes the current view
        refresh:function() {
            this.show(this.currentId, true);
        },
        //shows the supplied media item and updates the current tab
        // @id, the id of the item to show or 0 if no item should be shown
        // @force, true if the tab should be reloaded even if the same item already is displayed there.
        show: function (id, force) {
            if (id && id > 0) {
                //if we have a new id, reload the current frame
                if (force || this.currentId != id) {
                    this.loadTab(this.getSelectedTab(), id);
                }
                this.$noItemSelected.hide();
                this.$viewport.show(0, function () {
                    // todo maybe this should be placed in another backbone view and be fired by an event
                    $('#metadata-tabs').find('a').removeClass('inactive');
                });
            } else {
                this.$viewport.hide(0, function () {
                    // call destroy on the custom scrollbar
                    ($("#details-viewport").jScrollPane().data().jsp).destroy();
                    // todo maybe this should be placed in another backbone view and be fired by an event
                    // we also need to remove the event and add a return false event
                    $('#metadata-tabs').find('a').addClass('inactive');
                });
                this.$noItemSelected.show();
            }
        },
        fileDeleted: function () {
            this.show(null);
        },
        getSelectedTab: function () {
            var currentTab = "information";
            if ($.bbq.getState("infotab") == "metadata")
                currentTab = "metadata";
            return currentTab;
        },
        setSelectedTab: function (tab) {
            $.bbq.pushState({ infotab: tab });
            this.updateSelectedTab();
        },
        updateSelectedTab: function () {
            var currentTab = this.getSelectedTab();
            $("li a", this.$domobject).removeClass("selected");
            $("li a[data-name='" + currentTab + "']", this.$domobject).addClass("selected");
        },
        loadTab: function (tab, id) {
            var self = this;
            // add notification
            var loading = new IVNotification('Loading...');
            var action = "information";
            if (tab == "metadata") {
	            action = "metadata";
            }
           //if no id is specified, use the current one
            if (id === undefined)
                id = this.currentId;
            app.load({
                controller: 'library',
                action: action,
                data: { id: id },
                success: function (data) {
                    // remove loading notification
                    loading.remove();

                    self.$viewport.html(data);
                    self.setSelectedTab(action);
                    self.currentId = id;

                    //Vertically center images (Imagine a world without ie... *sigh*)
                    var $img = $('#image-container', self.$domobject).find('img');
                    if ($img.length > 0) {
                        var imgHeight = $img.height();
                        var spanHeight = $('#image-container', self.$domobject).height();
                        var margin = ((spanHeight - imgHeight) / 2);
                        $img.css('margin-top', margin);
                    }
                },
                complete: function () {
                    self.initViewport();
                },
                error: function (error) {
                    loading.remove();
                    app.showError("Error loading information", error);
                }
            });
        },
        initViewport: function () {
            var self = this;
            app.log('init');
            //Calculate height och scrollable area            
            var height = self.$domobject.height() - $('#metadata-tabs', self.$domobject).height();
            $("#details-viewport").height(height - 20);
            $("#details-viewport").jScrollPane();
        }
    });
    ns.InfoPanel = infoPanel;
})(jQuery, window.ImageVaultUi = window.ImageVaultUi || {});