﻿(function ($, ns, undefined) {
    var libraryView = function (config) {
        this.init(config);
        return this;
    };
    libraryView.prototype = {
        //Properties
        isLoading: true,
        currentPageNo: 0,
        pageSize: 100,
        lastCount: null,
        maxCount: 50,
        ivUiAjax: null,
        //Methods
        onSearchFailed: null,
        vaults: null,
        nav: null,
        toolbar: null,
        bottomBar: null,
        infopanel: null,
        thumbnailFormatId: null,
        insertMultiple: null,
        //methods
        init: function (config) {
            //map external functions
            $.extend(this, config);
            //#stream div is recreated by jscrollpane and cannot be cached...
            //var $stream = $("#main #streamGenerator");
            var self = this;
            //Create instances

            this.nav = new ImageVaultUi.Nav({});
            this.toolbar = new ImageVaultUi.Toolbar({ $domobject: $("#toolbar"), thumbnailFormatId: self.thumbnailFormatId });
            this.bottomBar = new ImageVaultUi.BottomBar({});
            this.lightbox = new ImageVaultUi.Lightbox({ $domobject: $("#lightbox"), thumbnailFormatId: self.thumbnailFormatId, insertMultiple: self.insertMultiple, maxCount: self.maxCount });
            this.infoPanel = new ImageVaultUi.InfoPanel({ $domobject: $("#details") });
            //Event for opening admin dialog
            $('#admin-anchor').click(function () {
                $.bbq.pushState({ 'admin-tab': 'rights' });
                ImageVaultUi.AdminView.OpenDialog();
                return false;
            });
            //Trigger event when item is clicked
            $('#stream li').live('click', function () {
                $(this).addClass('selected');
                var id = $(this).attr('data-item-id');
                //update infopanel
                self.infoPanel.show(id);
                //Push state to hash and let the hashchange event do the rest
                var state = {};
                state['item'] = id;
                $.bbq.pushState(state);
            });
            app.bind('media:versionchange', function() { self.performSearch(0) });
            //Event triggers when a url hash fragment changes
            $(window).bind('hashchange', function (e) {
                $('#SearchQuery').val($.bbq.getState('q'));
                //Check if a search should be performed
                if (app.hasStateChanged('vaultid') ||
                    app.hasStateChanged('categories') ||
                    app.hasStateChanged('q') ||
                    app.hasStateChanged('order') ||
                    app.hasStateChanged('direction')) {
                    //Perform search based on hash fragments
                    self.performSearch(0);
                }
                //Save current state so we can check if something has changed next time
                app.set({ previousState: e });
                //store the change in the hash to session
                if (window.sessionStorage) {
                    window.sessionStorage.libraryHash = window.location.hash;
                    window.sessionStorage.currentUserName = app.get("currentUserName");
                }

                // Placeholder fix for IE users ...
                $('#SearchQuery').labelize();
                
            });

            //Search event
            $('#search form').submit(function () {
                //Push state to hash
                $.bbq.pushState({ 'q': $('#SearchQuery').val() });
                return false;
            });

            // attach event for double clicking preview
            $('#stream li img').live('dblclick', function () {
                self.loadPreview();
            });
            // attach event for single clicking the preview icon
            $('#stream li.selected span.preview').live('click', function () {
                self.loadPreview();
            });
            $('#stream li input').live('change', function () {
                var $checkbox = $(this);
                var checked = $checkbox.is(":checked");
                var id = $checkbox.closest("li").attr("data-item-id");
                app.trigger("stream:item" + (checked ? "Checked" : "Unchecked"), id);
                return false;
            });


            //Resize stream height on window resize 
            $(window).resize(function () {
                self.initViewport();
            });
            app.bind("lightbox:itemRemoved", this.removeItemChecked, this);
            app.bind("lightbox:itemsCleared", this.removeItemChecked, this);
            app.bind("lightbox:itemAdded", this.lightboxItemAdded, this);
            app.bind("share:created",this.shareCreated,this);
            app.bind("share:removed", this.shareRemoved, this);

            //if we have stored an old hash in the session, navigate to it
            if (window.sessionStorage && window.sessionStorage.libraryHash) {
                //only reuse session storage for the same user that created it.
                if (window.sessionStorage.currentUserName == app.get("currentUserName")) {
                    window.location.hash = window.sessionStorage.libraryHash;
                }
            }

        },
        //when an item is removed from lightbox, deselect it from stream as well...
        removeItemChecked: function (id) {
            if (id === undefined) {
                $("#stream li input:checked").removeAttr('checked').parent().fadeOut('fast');
            } else {
                $("#stream li[data-item-id='" + id + "'] input:checked").removeAttr('checked').parent().fadeOut('fast');
            }
        },
        //makes sure that the checkbox state is correct
        lightboxItemAdded: function (id) {
            var $checkbox = $("#stream li[data-item-id='" + id + "'] input:checkbox");
            var checked = $checkbox.is(":checked");
            if (checked)
                return;
            $checkbox.attr('checked', "checked").parent().show();
        },
        //add the shared class to the mediaItem node when share is created
        shareCreated:function(mediaItemIds) {
            for (var i = 0; i < mediaItemIds.length; i++) {
                var id = mediaItemIds[i];
                $("#stream li[data-item-id='" + id + "']").addClass("shared");
            }
        	// Refresh the Information-tab
            if (this.infoPanel.getSelectedTab() == "information")
                this.infoPanel.refresh();
        },
    	//remove the shared class from the mediaItems that doesn't belong to any share
        shareRemoved: function (mediaItemIds) {
        	for (var i = 0; i < mediaItemIds.length; i++) {
        		var id = mediaItemIds[i];
        		$("#stream li[data-item-id='" + id + "']").removeClass("shared");
			}
        	// Refresh the Information-tab
        	if (this.infoPanel.getSelectedTab() == "information")
        		this.infoPanel.refresh();
        },
    	//Perform search based on url hash fragments
        performSearch: function (page) {
            var self = this;
            var searchString = $.bbq.getState('q');
            var vaultId = $.bbq.getState('vaultid');
            var categories = this.nav.navCategoriesTab.getSelectedCategories();
            var sortOrder = $.bbq.getState('order');
            var sortDirection = $.bbq.getState('direction');

            //Create model parameters to pass to server...
            var params = { model: { SearchQuery: searchString, SelectedVault: { Id: vaultId }, SelectedCategoryId: categories, SelectedSortOrder: { Order: sortOrder, Direction: sortDirection }, PageNo: page} };
            //...as JSON
            var json = JSON.stringify(params);

            //Notify user that we are getting stuff from server
            var loading = new IVNotification('Loading...');
            self.isLoading = true;
            if (page == 0) {
                $('#stream ul').fadeTo('normal', 0.5);
            }

            self.ivUiAjax = app.load({
                url: 'library/search',
                data: json,
                success: function (data) {
                    var $data = $(data);
                    self.lastCount = $data.find('li').length;
                    //We return one extra item to see if there is more
                    if (self.lastCount > self.pageSize) {
                        //Remove last item
                        $data.find('li').last().remove();
                    }
                    var api = $('#stream').data('jsp');
                    //Insert partial view from server to dom.
                    if (page > 0) {
                        //Append li:s to existing ul
                        $('#stream ul').append($data.find('li'));
                        //Reinititialize scrollbars
                        if (typeof api != 'undefined') {
                            api.reinitialise();
                        }
                    } else {
                        //Reset page number
                        self.currentPageNo = 0;
                        if (typeof api != 'undefined') {
                            api.destroy();
                        }
                        //Insert ul to stream
                        $('#stream').html($data);
                        //Init scrollbars etc on viewport
                        self.initViewport();
                    }
                    //Set up events etc on thumbnails
                    self.initThumbnails();

                    //Update toolbar
                    var itemId = $.bbq.getState('item');
                    var selectedItemIsInSearchSet = false;
                    if (typeof itemId != 'undefined') {
                        var selectedItem = $('#stream li[data-item-id=' + itemId + ']');
                        if (selectedItem.length > 0) {
                            selectedItemIsInSearchSet = true;
                            selectedItem.addClass('selected');
                        }
                    }
                    //update infopanel
                    if (selectedItemIsInSearchSet) {
                        self.infoPanel.show(itemId);
                    } else {
                        self.infoPanel.show(0);
                    }
                    self.toolbar.updateToolbar();

                    //Remove loading notification
                    loading.remove();
                    self.isLoading = false;
                },
                error: function (error) {
                    loading.remove();
                    app.showError("Error performing search", error);
                }
            });
        },
        //Init events etc for thumbnails in stream
        initThumbnails: function () {
            //Center images
            $('#stream img').each(function () {
                var $img = $(this);
                var $oDiv = $img.closest("div");
                var id = $img.closest("li").attr("data-item-id");
                $img.draggable({
                    appendTo: "body",
                    scope: "mediaItem",
                    helper: function () {
                        var itemId = $(this).closest('figure').attr('data-item-id');
                        return $('<div class="clone"><img src="' + $(this).attr('src') + '" data-item-id="' + itemId + '" /></div>');
                    },
                    zIndex: 1000,
                    opacity: 0.9,
                    containment: "body",
                    //					cursor:"crosshair",
                    start: function (event, ui) {
                        app.log("start drag: id:" + id + "," + event + "," + ui);
                        ui.helper.attr("data-item-id", id);
                    }
                });
                if (!$img.attr('height')) {
                    $oDiv.css('margin-top', 20);
                    return;
                }
                //Vertically center images (Imagine a world without ie... *sigh*)
                var imgHeight = $img.height();
                var divHeight = $img.closest('.thumbnail').height();
                var margin = ((divHeight - imgHeight) / 2) - 2;
                $oDiv.css('margin-top', margin);
            });
        },
        //Init scrollbars etc
        initViewport: function () {
            var self = this;
            //Calculate height of scrollable area
            var height = $('#main').height() - 50;
            $('#stream').height(height);
            //Init scrollbars
            $('#stream').jScrollPane();

            //Some obscure calculating for shadow effect. TODO
            /*var width = ((Math.floor($('#stream').width() / 170)) * 170) + 40;
            $('#stream .shadow').width(width);*/

            //Event triggered when scrolling
            $('#stream').bind('jsp-scroll-y', function (event, scrollPositionY/*, isAtBottom, isAtTop*/) {
                //Check if there is more stuff to get
                if (self.lastCount <= self.pageSize) { return; }
                var contentHeight = $('#stream ul').height();
                var contentPosition = scrollPositionY + $('#stream').height();
                //Load more when reaching 200 px from bottom
                if (contentPosition > contentHeight - 200) {
                    if (self.isLoading) { return; }
                    self.currentPageNo = self.currentPageNo + 1;
                    self.performSearch(self.currentPageNo);
                }
            });

            $('#stream li span.select input:checked').parent().show();

            $('#stream li').hoverIntent(function () {
                $(this).find('span.select, span.preview').fadeIn('fast');
            }, function () {
                $(this).find('span.preview').fadeOut('fast');
                if (!$(this).find('span.select input').is(':checked')) {
                    $(this).find('span.select').fadeOut('fast');
                }
            });
        },
        loadPreview: function () {
            // retrieve the current item
            var itemId = $.bbq.getState('item');
            // add notification
            var loading = new IVNotification('Loading...');
            app.load({
                controller: 'library',
                action: 'Preview',
                data: { mediaItemId: itemId },
                success: function (data) {
                    // remove loading notification
                    loading.remove();
                    // append preview to stream
                    $('body').append(data);
                    // retrieve the width from the width attribute
                    var width = $('#preview-pane img').attr('width');
                    // retrieve the height from the height attribute
                    var height = $('#preview-pane img').attr('height');
                    // add padding
                    height = parseInt(height) + 34;
                    // position the preview correctly in the stream
                    $('#preview-pane')
                        .css('width', width)
                        .css('height', height)
                        .center(false);
                    // prevent from closing
                    $('#preview-pane').click(function (event) {
                        event.stopPropagation();
                    });
                    // attach keyup command for closing the preview
                    $(document).keyup(function (e) {
                        if (e.keyCode == 27) { $('#preview-pane .close').click(); }   // esc
                    });
                    // attach body click for closing the preview
                    $('body').click(function () {
                        $('#preview-pane .close').click(); // body click
                    });
                    // close preview pane
                    $('#preview-pane .close').click(function () {
                        $('#preview-pane').fadeOut('fast', function () {
                            $(this).remove();
                        });
                        return false;
                    });

                },
                error: function (error) {
                    loading.remove();
                    app.showError("Error loading preview", error);
                }
            });
        }
    };
    ns.LibraryView = libraryView;
})(jQuery, window.ImageVaultUi = window.ImageVaultUi || {});



