﻿(function ($, ns) {
    var lightbox = function (config) {
        this.init(config);
        return this;
    };
    lightbox.prototype = {
        //properties
        $domobject: null,
        $downloadLightboxMenuTemplate: null,
        leftPos: 0,
        thumbnailFormatId: null,
        insertMultiple: null,
        maxCount: 0,
        //methods
        init: function (config) {
            $.extend(this, config);

            if (this.$domobject === undefined)
                throw "$domobject is not defined in config";
            var self = this;

            this.$domobject.droppable({
                //only accept items with the data-item-id attribute > 0
                scope: "mediaItem",
                accept: "img",
                hoverClass: 'drophover',
                tolerance: "touch",
                over: function () {
                    $('.clone').addClass('over');
                    $('.clone').append('<img src="images/icon-add.png" class="helper" />');
                    app.log("over");
                },
                out: function () {
                    $('.clone').removeClass('over');
                    $('.clone').find('img:last-child').remove();
                    app.log("out");
                },
                drop: function (event, ui) {
                    //ui.helper.unwrap();
                    var id = ui.helper.attr("data-item-id");
                    app.log("drop " + id);
                    self.addToLightbox(id);
                }
            });
            // init scrollbars
            this.redrawLightbox();

            //Resize stream height on window resize 
            $(window).resize(function () {
                self.redrawLightbox();
            });

            this.$domobject.find('span.remove').live('click', function () {
                self.removeFromLightbox($(this).closest('li').attr('data-item-id'));
            });

            this.$domobject.find('a.clear').live('click', function () {
                if ($(this).hasClass("inactive"))
                    return false;
                self.clearLightbox();
                return false;
            });
            this.$domobject.find("a.insert").live('click', function () {
                if ($(this).hasClass("inactive"))
                    return false;
                self.insertMultipleItems();
                return false;
            });
            this.$domobject.find("a.download").live('click', function () {
                if ($(this).hasClass("inactive"))
                    return false;
                self.openDownloadMenu();
                return false;
            });
            this.$domobject.find("a.move").live('click', function() {
                if ($(this).hasClass("inactive"))
                    return false;
                if (!self.canMoveWholeLightbox()) {
                    self.showCannotMoveLightboxPopup();
                    return false;
                }
                self.openMoveMenu();
                return false;
            });
            this.$domobject.find("a.organize").live('click', function () {
                //disable anchor when button is disabled
                if ($(this).hasClass("inactive"))
                    return false;
                if (!self.canOrganizeWholeLightbox()) {
                    self.showCannotOrganizeLightboxPopup();
                    return false;
                }
                return true;
            });
            this.$domobject.find("a.share").live('click', function () {
            	if ($(this).hasClass('inactive')) { return false; }
            	self.openShareMenu();
            	return false;
            });
            this.$domobject.find('li').live('mouseover mouseout', function (event) {
                if (event.type == 'mouseover') {
                    $(this).find('span.remove').show();
                } else {
                    $(this).find('span.remove').hide();
                }
            });

            //listen for application events
            app.bind("stream:itemChecked", this.addToLightbox, this);
            app.bind("stream:itemUnchecked", this.removeFromLightbox, this);
            //monitor changes for opener changes
            app.bind("app:openedForInsertChanged", this.updateToolbar, this);
            app.bind("media:versionchange", this.reloadLightbox, this);
        },
        //redraws and initializes the lightbox
        redrawLightbox: function (html) {
            var pane = this.$domobject.find('.pane');
            var jspane = pane.find('.jspPane');
            this.leftPos = jspane.css('left');
            if (html) {
                this.$domobject.html(html);
                pane = this.$domobject.find('.pane');
            }

            this.initializejScrollPane(pane);
            jspane = pane.find('.jspPane');
            jspane.css('left', this.leftPos);

            app.log(this.leftPos);
            this.updateToolbar();

        },
        //checks if all items in the lightbox can be organized
        canOrganizeWholeLightbox: function () {
            //find all selected items
            var items = $(".pane li", this.$domobject);
            //find all items that can be organized
            var contributeItems = $(".pane li.contribute", this.$domobject);
            //can we organize all?
            return (items.length == contributeItems.length) && (this.maxCount == 0 | items.length <= this.maxCount);
        },
        showCannotOrganizeLightboxPopup: function () {
            var self = this;
            if (this.cannotOrganizePopup) {
                if (this.cannotOrganizePopup.isVisible()) {
                    this.cannotOrganizePopup.hide();
                    return false;
                } else {
                    //new id, throw item and get a new one
                    this.cannotOrganizePopup.get("$domobject").remove();
                }
            }

            //Calculate position for the menu
            var $organizeButton = $('#lightbox-actions .organize', self.$domobject);
            //Load template with data
            if (self.$cannotOrganizePopupTemplate == undefined)
                self.$cannotOrganizePopupTemplate = $('#cannot-organize-lightbox-menu-template');
            var $popup = self.$cannotOrganizePopupTemplate.tmpl()
            //hide it
				.css("display", "none")
            //Append template to action list item
				.appendTo($organizeButton.parent());
            //add event handlers for the yes/no buttons
            $("a[data-action='CancelOrganize']", $popup).click(function () {
                self.cannotOrganizePopup.hide();
                return false;
            });
            $("a[data-action='RemoveMediaItemsThatCannotBeOrganized']", $popup).click(function () {
                self.cannotOrganizePopup.hide();
                setTimeout(function () { self.removeNonOrganizableItems(); }, 400);
                return false;
            });

            var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup });
            popup.show();

            self.cannotOrganizePopup = popup;
        },
        removeNonOrganizableItems: function () {
            var self = this;
            var itemsToRemove = self.getMediaItemIdsThatCannotBeOrganized();
            self.removeFromLightbox(itemsToRemove);
        },
        getMediaItemIdsThatCannotBeOrganized: function () {
            var self = this;
            var arr = [];
            $(".pane li", self.$domobject).each(function () {
                if ($(this).hasClass("contribute"))
                    return;
                var id = parseInt($(this).attr("data-item-id"));
                arr.push(id);
            });
            return arr;
        },
        // checks if we can move all items in lightbox
        canMoveWholeLightbox: function () {
            //find all selected items
            var items = $(".pane li", this.$domobject);
            //find all items that can be moved
            var moveItems = $(".pane li.move", this.$domobject);
            //can we move all?
            return (items.length == moveItems.length) && (this.maxCount == 0 | items.length <= this.maxCount);
        },
        showCannotMoveLightboxPopup: function () {
            var self = this;
            if (this.cannotMovePopup) {
                if (this.cannotMovePopup.isVisible()) {
                    this.cannotMovePopup.hide();
                    return false;
                } else {
                    this.cannotMovePopup.get("$domobject").remove();
                }
            }

            //Calculate position for the menu
            var $moveButton = $('#lightbox-actions .move', self.$domobject);
            //Load template with data
            if (self.$cannotMovePopupTemplate == undefined)
                self.$cannotMovePopupTemplate = $('#cannot-move-lightbox-menu-template');
            var $popup = self.$cannotMovePopupTemplate.tmpl()
            //hide it
				.css("display", "none")
            //Append template to action list item
				.appendTo($moveButton.parent());
            //add event handlers for the yes/no buttons
            $("a[data-action='CancelMove']", $popup).click(function () {
                self.cannotMovePopup.hide();
                return false;
            });
            $("a[data-action='RemoveMediaItemsThatCannotBeMoved']", $popup).click(function () {
                self.cannotMovePopup.hide();
                setTimeout(function () { self.removeNonMoveableItemsAndDisplayMoveMenu(); }, 400);
                return false;
            });

            var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup });
            popup.show();

            self.cannotMovePopup = popup;
        },
        removeNonMoveableItemsAndDisplayMoveMenu: function () {
            var self = this;
            var itemsToRemove = self.getMediaItemIdsThatCannotBeMoved();
            self.removeFromLightbox(itemsToRemove);
            //open up the move menu again, if we have any items left...
            if ($(".pane li", self.$domobject).hasClass('move'))
                self.openMoveMenu();
        },
        getMediaItemIdsThatCannotBeMoved: function () {
            var self = this;
            var arr = [];
            $(".pane li", self.$domobject).each(function () {
                if ($(this).hasClass("move"))
                    return;
                var id = parseInt($(this).attr("data-item-id"));
                arr.push(id);
            });
            return arr;
        },
        reloadLightbox:function() {
            var self = this;
            app.load({
                controller: 'Library',
                action: 'GetLightbox',
                success: function (data) {
                    self.redrawLightbox(data);
                    self.updateMenuTemplates(data);
                }
            });
        },
        addToLightbox: function (itemId) {
            var self = this;
            var canBeMoved = $('#stream li[data-item-id=' + itemId + ']').hasClass('move');
            var canBeShared = $('#stream li[data-item-id=' + itemId + ']').hasClass('share');
            app.load({
                controller: 'Library',
                action: 'AddToLightBox',
                data: { mediaItemId: itemId, canBeMoved: canBeMoved, canBeShared: canBeShared },
                success: function (data) {
                    self.redrawLightbox(data);
                    self.updateMenuTemplates(data);
                    app.trigger("lightbox:itemAdded", itemId);
                }
            });
        },
        //updates the download, move and cannot-organize menu templates
        updateMenuTemplates: function (data) {
            var element = $(document.createElement("div")).html(data);
            this.$downloadLightboxMenuTemplate = $("#download-lightbox-menu-template", element);
            this.$cannotOrganizePopupTemplate = $("#cannot-organize-lightbox-menu-template", element);
            this.$cannotMovePopupTemplate = $("#cannot-move-lightbox-menu-template", element);
            this.$moveLightboxMenuTemplate = $("#move-lightbox-menu-template", element);
            this.$shareLightboxMenuTemplate = $("#share-lightbox-menu-template", element);
        },
        //updates the lightbox toolbar. Will be called whenever the lightbox contents changes
        updateToolbar: function () {
            var noOfItems = $('.pane li', this.$domobject).length;

            if (noOfItems == 0) {
                this.enableButton("insert", false);
                this.enableButton("organize", false);
                this.enableButton("download", false);
                this.enableButton("clear", false);
                this.enableButton("move", false);
                this.enableButton("share", false);
                return;
            }
            //insert is only enabled if we have insertMultiple set and we have a valid callback
            this.enableButton("insert", this.insertMultiple && app.openedForInsert());
            this.enableButton("organize");
            this.enableButton("download");
            this.enableButton("clear");
            this.enableButton("move", $('.pane li', this.$domobject).hasClass('move'));
            this.enableButton("share");
        },
        //enables or disables a toolbar button
        enableButton: function (name, enabled) {
            if (enabled === undefined)
                enabled = true;
            var $button = $('#lightbox-actions a.' + name, this.$domobject);
            if (enabled) {
                $button.removeClass("inactive");
            } else {
                $button.addClass("inactive");
            }
        },
        //removes one or more items from the lightbox.
        //@itemIds either an array of id or a single id of the item that should be removed
        //@callback Optional callback function that is invoked when the items is removed from the lightbox successfully.
        removeFromLightbox: function (itemIds, callback) {
            if (!itemIds)
                return;
            //make sure that it is an array
            if (!$.isArray(itemIds)) {
                itemIds = [itemIds];
            }
            var self = this;
            for (var i = 0; i < itemIds.length; i++) {
                var $figure = $('.pane li[data-item-id=' + itemIds[i] + ']', this.$domobject);
                $figure.remove();
                app.trigger("lightbox:itemRemoved", itemIds[i]);
            }

            self.redrawLightbox();

            //only needed for the remove button since all other replaces the html
            self.updateToolbar();
            app.load({
                url: 'Library/RemoveFromLightBox',
                data: { mediaItemIds: itemIds },
                success: function (data) {
                    self.updateMenuTemplates(data);
                    if (callback) {
                        callback();
                    }
                }
            });
        },
        clearLightbox: function () {
            var self = this;
            app.load("Library", "ClearLightBox", function (data) {
                self.redrawLightbox(data);
                app.trigger("lightbox:itemsCleared");
            });
        },
        openDownloadMenu: function () {
            var self = this;
            if (this.downloadPopup) {
                if (this.downloadPopup.isVisible()) {
                    this.downloadPopup.hide();
                    return false;
                } else {
                    //new id, throw item and get a new one
                    this.downloadPopup.get("$domobject").remove();
                }
            }
            //Display loading notification
            var loading = new IVNotification('Loading...');
            //Get available conversion formats for mediaitem from server 
            app.load({
                controller: 'library',
                action: 'GetAvailableMediaFormatsForLightbox',
                dataType: 'JSON',
                success: function (result) {
                    //Wrap the data to use with jquery template
                    var data = [{ 'format': result}];
                    //Calculate position for the menu
                    var $downloadButton = $('#lightbox-actions .download', self.$domobject);
                    //				var position = $downloadButton.width() / 2 - 100;
                    //Load template with data
                    if (self.$downloadLightboxMenuTemplate == undefined)
                        self.$downloadLightboxMenuTemplate = $('#download-lightbox-menu-template');
                    var $popup = self.$downloadLightboxMenuTemplate.tmpl(data)
                    //hide it
						.css("display", "none")
                    //Append template to action list item
						.appendTo($downloadButton.parent());

                    //add event handlers for the yes/no buttons
                    $("a[data-action='CancelDownload']", $popup).click(function () {
                        self.downloadPopup.hide();
                        return false;
                    });
                    $("a[data-action='RemoveMediaItemsThatCannotBeDownloaded']", $popup).click(function () {
                        self.downloadPopup.hide();
                        setTimeout(function () { self.removeUnavailableItemsAndDisplayDownloadMenu(); }, 400);
                        return false;
                    });

                    var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup });
                    popup.bind("downloadClick", function ($e) {
                        var mediaFormatId = $e.attr("data-id");
                        popup.hide();
                        window.location = app.createPath("library", "LightboxAsZipArchive/" + mediaFormatId);
                    });
                    popup.show();

                    self.downloadPopup = popup;
                    //We are done, remove loading
                    loading.remove();


                    var maxHeight = $("#stream").height() / 2;

                    $("#download-lightbox-scroll").css("max-height", maxHeight);
                    $("#download-lightbox-scroll").jScrollPane();

                    $("#download-lightbox-scroll .jspDrag").click(function (e) {
                        // prevent popup menu from closing when dragging the scrollbar
                        if (e.stopPropagation)
                            e.stopPropagation();
                        else
                            e.cancelBubble = true;
                    });
                },
                error: function (error) {
                    loading.remove();
                    app.showError("Error loading available MediaFormats for Lightbox", error);
                }
            });
            return true;

        },
        openMoveMenu: function () {
            var self = this;
            if (this.movePopup) {
                if (this.movePopup.isVisible()) {
                    // menu open, hide
                    this.movePopup.hide();
                    return false;
                } else {
                    //new id, throw item and get a new one
                    this.movePopup.get("$domobject").remove();
                }
            }
            var loading = new IVNotification('Loading...');

            app.load({
                controller: 'library',
                action: 'GetMoveableVaultsForLightbox',
                dataType: 'JSON',
                success: function (result) {
                    var data = [{ 'vault': result}];
                    if (self.$moveLightboxMenuTemplate == undefined)
                        self.$moveLightboxMenuTemplate = $('#move-lightbox-menu-template');
                    var $popup = self.$moveLightboxMenuTemplate.tmpl(data).css("display", "none").appendTo('#lightbox-actions .move');

                    var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup });
                    popup.bind("popup:linkClicked", function ($a) {
                        popup.hide();
                        window.location = $a.attr("href");
                        return false;
                    });

                    popup.show();
                    self.movePopup = popup;
                    loading.remove();

                    var maxHeight = $("#stream").height() / 2;

                    $("#move-lightbox-scroll").css("max-height", maxHeight);
                    $("#move-lightbox-scroll").jScrollPane();

                    $("#move-lightbox-scroll .jspDrag").click(function (e) {
                        // prevent popup menu from closing when dragging the scrollbar
                        if (e.stopPropagation)
                            e.stopPropagation();
                        else
                            e.cancelBubble = true;
                    });
                },
                error: function (error) {
                    loading.remove();
                    app.showError("Error loading moveable vaults for ligthbox", error);
                }
            });

            return true;
        },
        openShareMenu: function () {
        	var self = this;
        	if (this.sharePopup) {
        		if (this.sharePopup.isVisible()) {
        			this.sharePopup.hide();
        			return false;
        		} else {
        			//new id, throw item and get a new one
        			this.sharePopup.get("$domobject").remove();
        		}
        	}
        	//Display loading notification
        	var loading = new IVNotification('Loading...');
        	//Get available conversion formats for mediaitem from server 
        	app.load({
        		controller: 'library',
        		action: 'GetAvailableMediaFormatsForLightbox',
        		dataType: 'JSON',
        		success: function (result) {
        			//Wrap the data to use with jquery template
        			var data = [{ 'format': result }];

        			//Calculate position for the menu
        			var $shareButton = $('#lightbox-actions .share', self.$domobject);
        			//Load template with data
        			if (self.$shareLightboxMenuTemplate == undefined)
        				self.$shareLightboxMenuTemplate = $('#share-lightbox-menu-template');
        			var $popup = self.$shareLightboxMenuTemplate.tmpl(data)
                    //hide it
						.css("display", "none")
                    //Append template to action list item
						.appendTo($shareButton.parent());

        		    //add event handlers for the yes/no buttons
        			$("a[data-action='CancelShare']", $popup).click(function () {
        			    self.sharePopup.hide();
        			    return false;
        			});
        			$("a[data-action='RemoveMediaItemsThatCannotBeShared']", $popup).click(function () {
        			    self.sharePopup.hide();
        			    setTimeout(function () { self.removeUnavailableItemsAndDisplayShareMenu(); }, 400);
        			    return false;
        			});

        			//Prevent closing menu on click
        			var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup });
        			popup.bind("shareClick", function ($a) {
        				popup.hide();
        				var mediaIdArr = JSON.parse($a.attr('data-media-id'));
        				var formatId = $a.attr('data-format-id');
                        new ImageVaultUi.ShareDialog({ mediaFormatId: formatId, mediaItemIds: mediaIdArr}).open();
        				return false;
        			});
        			popup.show();

        			self.sharePopup = popup;
        			//We are done, remove loading
        			loading.remove();

        			var maxHeight = $("#stream").height() / 2;

        			$("#share-lightbox-scroll").css("max-height", maxHeight);
        			$("#share-lightbox-scroll").jScrollPane();

        			$("#share-lightbox-scroll .jspDrag").click(function (e) {
        				// prevent popup menu from closing when dragging the scrollbar
        				if (e.stopPropagation)
        					e.stopPropagation();
        				else
        					e.cancelBubble = true;
        			});
        		},
        		error: function (error) {
        			loading.remove();
        			app.showError("Error loading available formats for MediaItem", error);
        		}
        	});
        	return true;
        },
        insertMultipleItems: function () {
            var self = this;
            app.json({
                controller: 'library',
                action: 'GetLightboxPublishedMediaInformation',
                datatype: 'JSON',
                data: { mediaFormatId: self.thumbnailFormatId },
                success: function (data) {
                    try {
                        app.insertCallback(data);
                    } catch (err) {
                        alert("Cannot insert media.\n[Error:" + err + "]");
                    }
                }
            });
        },
        removeUnavailableItemsAndDisplayShareMenu: function () {
            var self = this;
            var itemsToRemove = self.getMediaItemIdsThatCannotBeUsed();
            self.removeFromLightbox(itemsToRemove, function() {
                //open up the share menu again, if we have any items left...
                if ($(".pane li", self.$domobject).length > 0)
                    self.openShareMenu();
            });
        },
       
        removeUnavailableItemsAndDisplayDownloadMenu: function () {
            var self = this;
            var itemsToRemove = self.getMediaItemIdsThatCannotBeUsed();
            self.removeFromLightbox(itemsToRemove, function (){
            //open up the download menu again, if we have any items left...
            if ($(".pane li", self.$domobject).length > 0)
                self.openDownloadMenu();
            });
        },
        getMediaItemIdsThatCannotBeUsed: function () {
            var self = this;
            var arr = [];
            $(".pane li.unavailable", self.$domobject).each(function () {
                var id = parseInt($(this).attr("data-item-id"));
                arr.push(id);
            });
            return arr;
        },
        initializejScrollPane: function (elm) {
            var self = this;
            elm.unbind('jsp-initialised');
            elm.bind('jsp-initialised', function (event, isScrollable) {
                //                app.log('isScrollable=' + isScrollable);
                if (!isScrollable) {
                    self.leftPos = 0;
                }
            }).jScrollPane({ showArrows: true });
        }
    };
    ns.Lightbox = lightbox;
})(jQuery, window.ImageVaultUi = window.ImageVaultUi || {});