﻿(function ($, ns, undefined) {
	var categoriesTab = function (config) {
		this.init(config);
		return this;
	};
	categoriesTab.prototype = {
		//properties
		nav: null,
		$domobject: undefined,
		previousCategoryState: null,
		//methods
		init: function (config) {
			$.extend(this, config);
			if (this.$domobject === undefined)
				throw "$domobject is not defined in config";

			var self = this;
			//Selected selected categories
			self.selectCategoriesFromHash();
			//Category tree
			$('li span', this.$domobject).live('click', function () {
				self.toggleCategoryChildren($(this));
				//Reset hight of viewport
				self.nav.initViewport();
			});
			//Event for category selection
			$('input', this.$domobject).live('change', function () {
				var categoryId = $(this).closest('li').attr('data-category-id');
				if ($(this).is(':checked')) {
					self.addCategoryToHash(categoryId);
				} else {
					self.removeCategoryFromHash(categoryId);
				}
			});
			$(window).bind('hashchange', function (e) {
				//Nothing has happend (with categories)
				if (e.getState('categories') == self.previousCategoryState) {
					return;
				}
				self.selectCategoriesFromHash();
				self.previousCategoryState = e.getState('categories');
			});
			//monitor changes to categories
			app.bind("category:added", this.refreshCategoryList, this);
			app.bind("category:deleted", this.refreshCategoryList, this);
			app.bind("category:edited", this.refreshCategoryList, this);
		},
		//called when the tab should be displayed
		show: function () {
			this.$domobject.show();
//			try {
				//this event can trigger exceptions in IE when starting up
				app.trigger("navtoolbar:disableButton", "edit");
//			}catch (e){}
		},
		//called when the tab should be hidden
		hide: function () {
			this.$domobject.hide();
		},

		/*************************************************
		* BBQ hash persistance methods
		*************************************************/
		//adds the category to the category hash
		addCategoryToHash: function (categoryId) {
			var categories = this.getSelectedCategories();
			//Add category id to array
			categories.push(categoryId);

			this.saveCategoriestToHash(categories);
		},
		removeCategoryFromHash: function (categoryId) {
			var categories = this.getSelectedCategories();
			//Remove category from array.
			for (var i = 0; i < categories.length; i++) {
				if (categories[i] == categoryId) {
					categories.splice(i, 1);
					break;
				}
			}
			this.saveCategoriestToHash(categories);
		},
		saveCategoriestToHash: function (categories) {
			//Remove key if no categories left
			if (categories.length == 0) {
				$.bbq.removeState('categories');
				return;
			}
			//Push state to hash
			var state = {};
			state['categories'] = categories.join('e');
			$.bbq.pushState(state);
		},
		//Gets selected categories from hash fragment and returns as array
		getSelectedCategories: function () {
			//using bbq
			var categoriesAsString = $.bbq.getState('categories');
			if (typeof categoriesAsString === "undefined" || categoriesAsString.length == 0) {
				return new Array();
			}
			return categoriesAsString.split('e');
			//alternate method using the input tags
			//			return $(">ul input:checked", this.$domobject).parent().map(function () {
			//				return $(this).attr("data-category-id");
			//			}).get();

		},

		toggleCategoryChildren: function ($element) {
			//Expand children
			if (!$element.hasClass('expanded')) {
				$element.addClass('expanded');
				$element.siblings('ul').show();
			}
			else {
				$element.removeClass('expanded');
				$element.siblings('ul').hide();
			}
		},

		selectCategoriesFromHash: function () {
			$('input', this.$domobject).removeAttr('checked');
			var categories = this.getSelectedCategories();
			for (var i in categories) {
				$('li[data-category-id=' + categories[i] + ']', this.$domobject).children('input').attr('checked', 'checked');
			}
		},
		//refreshes the category list
		refreshCategoryList: function () {
			var self = this;
			//store all expanded categories
			var expandedCategories = this.getExpandedCategories();
			var selectedCategories = this.getSelectedCategories();
			app.load({
				controller: 'library',
				action: 'RefreshCategoryList',
				success: function (data) {
					//update vault list
					self.$domobject.html(data);
					//remove any categories from selection that was deleted
					var availableCategories = self.getAvailableCategories();
					var removedSelectedCategories = _.difference(selectedCategories, availableCategories);
					for (var i = 0; i < removedSelectedCategories.length; i++) {
						self.removeCategoryFromHash(removedSelectedCategories[i]);
						app.log("removed selected category " + removedSelectedCategories[i] + " since it was deleted");
					}
					//restore expanded state
					self.expandCategories(expandedCategories);
					//select the categories that should be selected
					self.selectCategoriesFromHash();
				}
			});
		},
		//gets all available categories (id)
		getAvailableCategories: function () {
			return $(">ul li", this.$domobject).map(function () {
				return $(this).attr("data-category-id");
			}).get();
		},
		//gets all expanded categories(id)
		getExpandedCategories: function () {
			return $("ul ul:visible", this.$domobject).parent().map(function () {
				return $(this).attr("data-category-id");
			}).get();
		},
		//Expands the supplied categories (list of id)
		expandCategories: function (list) {
			for (var i = 0; i < list.length; i++) {
				this.expandCategory(list[i]);
			}
		},
		//expands the selected category (id)
		expandCategory: function (categoryId) {
			var $element = $("li[data-category-id='" + categoryId + "']>span");
			$element.addClass('expanded');
			$element.siblings('ul').show();
		}
	};
	ns.NavCategoriesTab = categoriesTab;
})(jQuery, window.ImageVaultUi = window.ImageVaultUi || {});