﻿(function ($, ns, undefined) {
	var navlibraryTab = function (config) {
		this.init(config);
		return this;
	};
	navlibraryTab.prototype = {
		//properties
		nav: null,
		$domobject: undefined,
		//methods
		init: function (config) {
			$.extend(this, config);
			if (this.$domobject === undefined)
				throw "$domobject is not defined in config";
			var self = this;
			


			//TODO: Rethink for collections etc...
			$('a', this.$domobject).live('click', function () {
				$(this).blur();
				var id = $(this).attr('data-vault-id');
				self.updateHash(id);
				return false;
			});
			app.bind("vaultDialog:vaultDeleted", this.vaultDeleted, this);
			app.bind("vaultDialog:vaultSaved", this.vaultSaved, this);

			$(window).bind('hashchange', function (e) {
				if (app.hasStateChanged('vaultid')) {
					self.selectVault(e.getState('vaultid'));
				}
			});
			//If no vault is selected. Select first. (all items)
			if (typeof $.bbq.getState('vaultid') === 'undefined') {
				$('a:first', this.$domobject).click();
			}
		},
		//shows the library tab
		show: function () {
			this.$domobject.show();
			var vaultId = $.bbq.getState("vaultid");

			if (vaultId > 0) {
				this.selectVault(vaultId);
			} else {				
				app.trigger("navtoolbar:disableButton", "edit");
			}
			app.bind("navtoolbar:editClick", this.openEditDialogForSelectedVault, this);
		},
		//hides the library tab
		hide: function () {
			this.$domobject.hide();
			app.unbind("navtoolbar:editClick", this.openEditDialogForSelectedVault, this);
		},
		selectVault: function (id) {
			var $anchor = $('a[data-vault-id=' + id + ']', this.$domobject);

			$('li', this.$domobject).removeClass('selected');
			$anchor.closest('li').addClass('selected');

			//Activate edit button if user is vault admin
			if ($anchor.attr('data-is-vault-admin') == "True") {
				app.trigger("navtoolbar:enableButton", "edit");
			} else {
				app.trigger("navtoolbar:disableButton", "edit");
			}
		},
		updateHash: function (id) {
			//Push state to hash
			$.bbq.pushState({ vaultid: id });
		},
		//opens the vault dialog for the current selected vault
		openEditDialogForSelectedVault: function () {
			var vaultId = this.getSelectedVaultId();
			ImageVaultUi.VaultDialog.OpenDialog(vaultId);
		},
		//opens the new vault dialog
		openNewVaultDialog: function () {
			ImageVaultUi.VaultDialog.OpenDialog(-1);
		},
		//callback methods for vault getting saved
		vaultSaved: function (vault) {
			this.refreshAndSelectVault(vault.Id);
		},
		//callback method for vault getting deleted
		vaultDeleted: function (vaultId) {
			this.removeVaultFromUi(vaultId);
		},
		//gets the selected vault id or 0 if no vault is selected
		getSelectedVaultId: function () {
			if (!$('li.selected a', this.$domobject).attr('data-vault-id')) {
				return 0;
			}
			return $('li.selected a', this.$domobject).attr('data-vault-id');
		},
		//checks if the user is admin for the selected vault
		getSelectedVaultIsAdmin: function () {
			if (!$('li.selected a', this.$domobject).attr('data-is-vault-admin')) {
				return false;
			}
			return $('li.selected a', this.$domobject).attr('data-is-vault-admin') == "True";
		},
		//marks the supplied vault id as selected
		markVaultAsSelected: function (vaultId) {
			$('a', this.$domobject).removeClass('selected');
			$('a[data-vault-id=' + vaultId + ']', this.$domobject).addClass('selected');
		},
		//removes a vault from the ui (in case of it is deleted)
		removeVaultFromUi: function (vaultId) {
			//remove vault from filter collection
			var liToRemove = $("a[data-vault-id='" + vaultId + "']", this.$domobject).parent();
			var previousLi = liToRemove.prev();
			//clear vault selection
			$('li', this.$domobject).removeClass('selected');
			//select previous vault
			previousLi.addClass('selected');
			liToRemove.remove();
			//update search hash
			$.bbq.removeState('vaultid');
		},
		//Refreshes the vault list and selects the vault with the given id.
		refreshAndSelectVault: function (vaultId) {
			var self = this;
			app.load({
				controller: 'library',
				action:'RefreshVaultList/' + vaultId,
				success: function (data) {
                    //force select the vaults tab
				    $.bbq.pushState({tab:"vaults"});
					//update vault list
					$("#vault-list", self.$domobject).html(data);
					//perform a click on the vault (will perform a search in case of a new vault)
					$("#vault-list a[data-vault-id='" + vaultId + "']", self.$domobject).click();
					//we need to update the search hash with force set to true since 
					//a rename should update the search scope to get the correct name
					//$.bbq.removeState('vaultid');
				}
			});
		}
	};
	ns.NavLibraryTab = navlibraryTab;
})(jQuery, window.ImageVaultUi = window.ImageVaultUi || {});