﻿(function ($, ns, undefined) {
    var toolbar = function (config) {
        this.init(config);
        return this;
    };
    toolbar.prototype = {
        downloadPopup: null,
        insertPopup: null,
        movePopup: null,
        sharePopup: null,
        thumbnailFormatId: null,
        init: function (config) {
            $.extend(this, config);
            var self = this;

            if (this.$domobject === undefined)
                throw "$domobject is not defined in config";

            $(window).bind('hashchange', function () {
                if (app.hasStateChanged('item')) {
                    //Triggered when the hash changes e.g. user clicks the back button in browser
                    self.updateToolbar();
                }
            });
            //append vaultid as querystring parameter when clicking on the import button
            $('a.import', this.$domobject).click(function () {
                var href = $(this).attr("href");
                var vaultid = $.bbq.getState('vaultid');
                if (vaultid !== undefined) {
                    href = href + "?vaultid=" + vaultid;
                }
                document.location = href;
                return false;
            });
            $('#actions .download', this.$domobject).click(function () {
                if ($(this).hasClass('inactive')) { return false; }
                self.openDownloadMenu($.bbq.getState('item'));
                return false;
            });
            $('#actions .insert', this.$domobject).click(function () {
                if ($(this).hasClass('inactive')) {
                    return false;
                }
                self.openInsertMenu($.bbq.getState('item'));
                return false;
            });
            $('#actions .delete', this.$domobject).click(function () {
                if ($(this).hasClass('inactive')) {
                    return false;
                }
                ImageVaultUi.DeleteDialog.OpenDialog($.bbq.getState('item'));
                return false;
            });
            $("#actions .organize", this.$domobject).click(function () {
                if ($(this).hasClass('inactive')) {
                    return false;
                }
                window.location.href = app.createPath("organize", "Item/" + $.bbq.getState('item'));
                return false;
            });
            $("#actions .move", this.$domobject).click(function () {
                if ($(this).hasClass('inactive')) {
                    return false;
                }
                self.openMoveMenu($.bbq.getState('item'));
                return false;
            });
            $("#actions .share", this.$domobject).click(function () {
                if ($(this).hasClass('inactive')) { return false; }
                self.openShareMenu($.bbq.getState('item'));
                return false;
            });
            $("#actions .more-button", this.$domobject)
                .click(function() {
                    if ($(this).hasClass('inactive')) {
                        return false;
                    }
                    var itemId = $.bbq.getState('item');
                    var $item = $('#stream li[data-item-id=' + itemId + ']');
                        self.openMoreMenu($item);
                    return false;
                });
            //monitor changes for opener changes
            app.bind("app:openedForInsertChanged", this.updateToolbar, this);
        },
        updateToolbar: function () {
            var id = $.bbq.getState('item');
            var $item = $('#stream li[data-item-id=' + id + ']');
            //If the item still is in the view, update toolbar.
            if ($item.length == 0) {
                this.enableButton("download", false);
                this.enableButton("insert", false);
                this.enableButton("delete", false);
                this.enableButton("organize", false);
                this.enableButton("move", false);
                this.enableButton("share", false);
                this.enableButton("more-button", false);
                return;
            }
            $item.addClass('selected').siblings().removeClass('selected');
            this.enableButton("delete", $item.hasClass('contribute'));
            this.enableButton("insert", $item.hasClass('insert') && app.openedForInsert());
            this.enableButton("download", $item.hasClass('download'));
            this.enableButton("organize", $item.hasClass('contribute'));
            this.enableButton("move", $item.hasClass('move'));
            this.enableButton("share", $item.hasClass('share'));
            this.enableButton("more-button", true);
        },
        //enables or disables a toolbar button
        enableButton: function (name, enabled) {
            if (enabled === undefined)
                enabled = false;
            var $button = $('#actions .' + name, this.$domobject);
            if (enabled) {
                $button.removeClass("inactive");
            } else {
                $button.addClass("inactive");
            }
        },
        openDownloadMenu: function (id) {
            var self = this;
            if (this.downloadPopup) {
                if (this.downloadPopup.isVisible()) {
                    this.downloadPopup.hide();
                    return false;
                } else {
                    //new id, throw item and get a new one
                    this.downloadPopup.get("$domobject").remove();
                }
            }
            //Display loading notification
            var loading = new IVNotification('Loading...');
            //Get available conversion formats for mediaitem from server 
            app.load({
                controller: 'library',
                action: 'GetAvailableFormatsForMediaItem',
                data: { 'mediaItemId': id },
                dataType: 'JSON',
                success: function (result) {
                    //Wrap the data to use with jquery template
                    var data = [{ 'format': result}];
                    //Calculate position for the menu
                    var position = $('#actions .download').width() / 2 - 100;
                    //Load template with data
                    var $popup = $('#download-menu-template').tmpl(data)
                    //hide it
                        .css("display", "none")
                    //Append template to action list item
                        .appendTo($('#actions .download'))
                    //Position menu
                        .css('left', position);
                    //Prevent closing menu on click
                    //.click(function (e) { e.stopPropagation(); $('#download-menu').fadeOut('fast', function () { $(this).remove(); }); });
                    var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup, MediaItemId: id });
                    popup.bind("popup:linkClicked", function ($a) {
                        popup.hide();
                        window.location = $a.attr("href");
                        return false;
                    });
                    popup.show();

                    self.downloadPopup = popup;
                    //We are done, remove loading
                    loading.remove();

                    var maxHeight = $("#stream").height() / 2;

                    $("#download-action-scroll").css("max-height", maxHeight);
                    $("#download-action-scroll").jScrollPane();

                    $("#download-action-scroll .jspDrag").click(function (e) {
                        // prevent popup menu from closing when dragging the scrollbar
                        if (e.stopPropagation)
                            e.stopPropagation();
                        else
                            e.cancelBubble = true;
                    });
                },
                error: function (error) {
                    loading.remove();
                    app.showError("Error loading available formats for MediaItem", error);
                }
            });
            return true;
        },
        openMoveMenu: function (itemId) {
            var self = this;
            if (this.movePopup) {
                if (this.movePopup.isVisible()) {
                    // menu open, hide
                    this.movePopup.hide();
                    return false;
                } else {
                    this.movePopup.get("$domobject").remove();
                }
            }
            var loading = new IVNotification('Loading...');

            app.load({
                controller: 'library',
                action: 'GetMoveableVaults',
                data: { 'mediaItemId': itemId },
                dataType: 'JSON',
                success: function (result) {
                    var data = [{ 'vault': result, 'itemId': itemId}];
                    var position = $('#actions .move').width() / 2 - 100;
                    var $popup = $('#move-menu-template').tmpl(data).css("display", "none").appendTo('#actions .move').css('left', position);

                    var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup, MediaItemId: itemId });
                    popup.bind("popup:linkClicked", function ($a) {
                        popup.hide();
                        window.location = $a.attr("href");
                        return false;
                    });

                    popup.show();
                    self.movePopup = popup;
                    loading.remove();

                    var maxHeight = $("#stream").height() / 2;

                    $("#move-action-scroll").css("max-height", maxHeight);
                    $("#move-action-scroll").jScrollPane();

                    $("#move-action-scroll .jspDrag").click(function (e) {
                        // prevent popup menu from closing when dragging the scrollbar
                        if (e.stopPropagation)
                            e.stopPropagation();
                        else
                            e.cancelBubble = true;
                    });
                },
                error: function (error) {
                    loading.remove();
                    app.showError("Error loading moveable vaults", error);
                }
            });

            return true;
        },
        openInsertMenu: function (id) {
            // if format id is specified, don't open the insert menu
            if (this.thumbnailFormatId) {
                return this.insertItem(id, this.thumbnailFormatId);
            }

            if (this.insertPopup && this.insertPopup.isVisible()) {
                this.insertPopup.hide();
                return false;
            }
            if (this.insertPopup) {
                if (this.insertPopup.isVisible()) {
                    this.insertPopup.hide();
                    return false;
                    //same item, just show it
                } else if (id == this.insertPopup.get("MediaItemId")) {
                    this.insertPopup.show();
                    return false;
                } else {
                    //new id, throw item and get a new one
                    this.insertPopup.get("$domobject").remove();
                }
            }

            var self = this;
            var loading = new IVNotification('Loading...');
            app.load({
                controller: 'library',
                action: 'GetAvailableFormatsForMediaItem',
                data: { 'mediaItemId': id },
                dataType: 'JSON',
                success: function (result) {
                    var data = [{ 'format': result}];
                    var position = $('#actions .insert').width() / 2 - 100;
                    var $popup = $('#insert-menu-template').tmpl(data)
                        .appendTo($('#actions .insert'))
                    //hide it
                        .css("display", "none")
                        .css('left', position);
                    //.click(function (e) { e.stopPropagation(); });
                    var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup, MediaItemId: id });
                    popup.bind("insertClick", function ($link) {
                        var mediaId = $link.attr('data-media-id');
                        var formatId = $link.attr('data-format-id');
                        self.insertItem(mediaId, formatId);
                        popup.hide();
                    });
                    popup.show();
                    self.downloadPopup = popup;

                    loading.remove();
                },
                error: function (error) {
                    loading.remove();
                    app.showError("Error loading available formats for MediaItem", error);
                }
            });
            return true;
        },
        insertItem: function (itemId, cfId) {
            app.json({
                controller: 'library',
                action: 'GetPublishedMediaItemInformation',
                datatype: 'JSON',
                data: { mediaItemId: itemId, mediaFormatId: cfId },
                success: function (data) {
                    try {
                        app.insertCallback(data);
                    } catch (err) {
                        alert("Cannot insert media.\n[Error:" + err + "]");
                    }
                }
            });
        },
        openShareMenu: function (id) {
            var self = this;
            if (this.sharePopup) {
                if (this.sharePopup.isVisible()) {
                    this.sharePopup.hide();
                    return false;
                } else {
                    //new id, throw item and get a new one
                    this.sharePopup.get("$domobject").remove();
                }
            }
            //Display loading notification
            var loading = new IVNotification('Loading...');
            //Get available conversion formats for mediaitem from server 
            app.load({
                controller: 'library',
                action: 'GetAvailableFormatsForMediaItem',
                data: { 'mediaItemId': id },
                dataType: 'JSON',
                success: function (result) {
                    //Wrap the data to use with jquery template
                    var data = [{ 'format': result }];
                    //Calculate position for the menu
                    var position = $('#actions .share').width() / 2 - 100;
                    //Load template with data
                    var $popup = $('#share-menu-template').tmpl(data)
                    //hide it
                        .css("display", "none")
                    //Append template to action list item
                        .appendTo($('#actions .share'))
                    //Position menu
                        .css('left', position);
                    //Prevent closing menu on click
                    //.click(function (e) { e.stopPropagation(); $('#download-menu').fadeOut('fast', function () { $(this).remove(); }); });
                    var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup, MediaItemId: id });
                    popup.bind("shareClick", function ($a) {
                        popup.hide();
                        var mediaId = $a.attr('data-media-id');
                        var formatId = $a.attr('data-format-id');
                        new ImageVaultUi.ShareDialog({ mediaFormatId: formatId, mediaItemIds: [mediaId]}).open();
                        return false;
                    });
                    popup.show();

                    self.sharePopup = popup;
                    //We are done, remove loading
                    loading.remove();

                    var maxHeight = $("#stream").height() / 2;

                    $("#share-action-scroll").css("max-height", maxHeight);
                    $("#share-action-scroll").jScrollPane();

                    $("#share-action-scroll .jspDrag").click(function (e) {
                        // prevent popup menu from closing when dragging the scrollbar
                        if (e.stopPropagation)
                            e.stopPropagation();
                        else
                            e.cancelBubble = true;
                    });
                },
                error: function (error) {
                    loading.remove();
                    app.showError("Error loading available formats for MediaItem", error);
                }
            });
            return true;
        },
        openMoreMenu: function($item) {
            var self = this;
            var id = $item.attr('data-item-id');

            if (this.morePopup) {
                if (this.morePopup.isVisible()) {
                    // menu open, hide
                    this.morePopup.hide();
                    return false;
                } else {
                    this.morePopup.get("$domobject").remove();
                }
            }

            var data = { 'id': id, 'version': true, 'shares': false };
            if ($item.hasClass('shared')) {
                data.shares = true;
            }
            var position = $('#actions .more-button').width() / 2 - 100;
            var $popup = $('#more-menu-template')
                .tmpl(data)
                .css('display', 'none')
                .appendTo('#actions .more-button')
                .css('left', position);

            var popup = new ImageVaultUi.PopupMenu({ $domobject: $popup, MediaItemId: id });
            popup.bind('popup:linkClicked', function ($a) {
                    popup.hide();
                    // handle different menu items
                    switch($a.attr('id')) {
                        case 'more-shares':
                            (new ImageVaultUi.ViewSharesDialog({ mediaItemId: parseInt($a.attr('data-media-id')) })).open();
                            break;
                        case 'more-versions':
                            (new ImageVaultUi.VersionsDialog({ mediaItemId: parseInt($a.attr('data-media-id')) })).open();
                            break;
                        default:
                            break;
                    }
                return false;
            });

            popup.show();
            self.morePopup = popup;
        }
    };
    ns.Toolbar = toolbar;
})(jQuery, window.ImageVaultUi = window.ImageVaultUi || {});